import logging
from typing import Optional

from scale_gp_beta.types import SpanType

from .span import Span, BaseSpan, NoOpSpan
from .util import is_disabled
from .scope import Scope
from .trace import Trace, BaseTrace, NoOpTrace
from .types import SpanInputParam, SpanOutputParam, SpanMetadataParam
from .trace_queue_manager import TraceQueueManager, tracing_queue_manager

log: logging.Logger = logging.getLogger(__name__)


def current_span() -> Optional[BaseSpan]:
    """Retrieves the currently active span from the execution context.

    This function relies on `contextvars` to manage the active span in
    a context-local manner, making it safe for concurrent execution
    environments (e.g., threads, asyncio tasks).

    Returns:
        Optional[BaseSpan]: The current BaseSpan instance if one is active,
                            otherwise None. This could be a 'Span' or 'NoOpSpan'.
    """
    return Scope.get_current_span()


def current_trace() -> Optional[BaseTrace]:
    """Retrieves the currently active trace from the execution context.

    Similarly, to `current_span()`, this uses `contextvars` for context-local
    trace management.

    Returns:
        Optional[BaseTrace]: The current BaseTrace instance if one is active,
                             otherwise None. This could be a 'Trace' or 'NoOpTrace'.
    """
    return Scope.get_current_trace()


def flush_queue() -> None:
    """
    Blocking flush of all requests in the queue.

    Useful for distributed applications to ensure spans have been committed before continuing.
    :return:
    """
    queue_manager = tracing_queue_manager()
    queue_manager.flush_queue()


def create_trace(
        name: str,
        span_type: SpanType = "TRACER",
        input: Optional[SpanInputParam] = None,
        output: Optional[SpanOutputParam] = None,
        metadata: Optional[SpanMetadataParam] = None,
        span_id: Optional[str] = None,
        trace_id: Optional[str] = None,
        group_id: Optional[str] = None,
        queue_manager: Optional[TraceQueueManager] = None,
) -> BaseTrace:
    """Creates a new trace and root span instance.

    A trace represents a single, logical operation or workflow. It groups multiple
    spans together. If tracing is disabled (via the 'DISABLE_SCALE_TRACING'
    environment variable), a `NoOpTrace` instance is returned which performs no
    actual tracing operations.

    When a trace is started (e.g., by using it as a context manager or calling its
    `start()` method), it becomes the `current_trace()` in the active scope.
    Similarly, the root span instance becomes the `current_span()` in the active
    scope.

    Args:
        name: The name of the trace.
        span_type (Optional[SpanType]): Type of root span.
        input (Optional[SpanInputParam]): Input of root span.
        output (Optional[SpanOutputParam]): Output of root span.
        metadata (Optional[SpanMetadataParam]): An optional, user-defined metadata.
        span_id (Optional[str]): An optional, user-defined ID for the root span.
                                 Max length is 38 characters.
        trace_id (Optional[str]): An optional, user-defined ID for the trace.
                                  If None, a unique trace ID will be generated.
                                  Max length is 38 characters.
        group_id (Optional[str]): An optional, id to group traces.
        queue_manager (Optional[TraceQueueManager], optional): An optional `TraceQueueManager`.
            Useful for when you need explicit control of flushing and client behavior.

    Returns:
        BaseTrace: A `Trace` instance if tracing is enabled, or a `NoOpTrace`
                   instance if tracing is disabled.
    """
    impl_input: SpanInputParam = input or {}
    impl_output: SpanOutputParam = output or {}
    impl_metadata: SpanMetadataParam = metadata or {}

    if is_disabled():
        log.debug(f"Tracing is disabled. Not creating a new trace.")
        return NoOpTrace(
            name=name,
            trace_id=trace_id,
            span_id=span_id,
            group_id=group_id,
            span_type=span_type,
            input=impl_input,
            output=impl_output,
            metadata=impl_metadata,
        )

    active_trace = current_trace()
    if active_trace is not None:
        log.warning(f"Trace with id {active_trace.trace_id} is already active. Creating a new trace anyways.")

    queue_manager = queue_manager or tracing_queue_manager()
    trace = Trace(
        name=name,
        trace_id=trace_id,
        span_id=span_id,
        group_id=group_id,
        queue_manager=queue_manager,
        span_type=span_type,
        input=impl_input,
        output=impl_output,
        metadata=impl_metadata,
    )
    log.debug(f"Created new trace: {trace.trace_id}")

    return trace


def create_span(
    name: str,
    span_type: SpanType = "STANDALONE",
    input: Optional[SpanInputParam] = None,
    output: Optional[SpanOutputParam] = None,
    metadata: Optional[SpanMetadataParam] = None,
    span_id: Optional[str] = None,
    parent_id: Optional[str] = None,
    trace_id: Optional[str] = None,
    group_id: Optional[str] = None,
    queue_manager: Optional[TraceQueueManager] = None,
) -> BaseSpan:
    """Creates a new span instance.

    A span represents a single unit of work or operation within a trace. Spans
    can be nested to form a hierarchy.

    If tracing is disabled (via 'DISABLE_SCALE_TRACING' environment variable),
    a `NoOpSpan` is returned. Additionally, if no explicit `parent` (Trace or Span)
    is provided and there is no `current_trace()` active in the scope, a `NoOpSpan`
    will also be returned to prevent orphaned spans.

    When a span is started (e.g., via context manager or `start()`), it becomes
    the `current_span()` in the active scope.

    If explicitly setting 'parent_id' and 'trace_id', ensure that the parent span
    has the same 'trace_id'.

    Args:
        name (str): A descriptive name for the span (e.g., "database_query",
                    "http_request").
        span_type (SpanType): The type of the span.
        input (Optional[SpanInputParam], optional): A dictionary containing
            input data or parameters relevant to this span's operation. Defaults to None.
        output (Optional[SpanOutputParam], optional): A dictionary containing
            output data or results from this span's operation. Defaults to None.
        metadata (Optional[SpanMetadataParam], optional):
            A dictionary for arbitrary key-value pairs providing additional
            context or annotations for the span. Values should be simple types.
            Defaults to None.
        span_id (Optional[str]): An optional, user-defined ID for the span.
        parent_id (Optional[str], optional): A `Span` id. Used for explicit control.
        Defaults to span id fetched from the active scope.
        trace_id (Optional[str], optional): A `Trace` id. Used for explicit control.
            Default to trace id fetched from the active scope.
        group_id (Optional[str]): An optional, id to group traces.
        queue_manager (Optional[TraceQueueManager], optional): An optional `TraceQueueManager`.
            Useful for when you need explicit control of flushing and client behavior.

    Returns:
        BaseSpan: A `Span` instance if tracing is enabled and a valid trace context
                  exists, or a `NoOpSpan` otherwise.
    """
    impl_input: SpanInputParam = input or {}
    impl_output: SpanOutputParam = output or {}
    impl_metadata: SpanMetadataParam = metadata or {}

    scoped_trace = current_trace()
    scoped_trace_id = scoped_trace.trace_id if scoped_trace else None
    scoped_group_id = scoped_trace.group_id if scoped_trace else None
    scoped_span = current_span()
    scoped_span_id = scoped_span.span_id if scoped_span else None

    parent_span_id: Optional[str] = parent_id or scoped_span_id
    # TODO: preference should be trace_id -> trace_id from parent span if parent_id present -> scoped_trace_id
    # group_id -> group_id from trace if trace_id specified -> group_id from span if span_id -> scoped_group_id
    impl_trace_id: Optional[str] = trace_id or scoped_trace_id
    impl_group_id: Optional[str] = group_id or scoped_group_id

    # TODO: do a check to ensure trace_id of parent_span matches trace_id (when trace_id is specified)

    noop_span = NoOpSpan(
        name=name,
        span_id=span_id,
        parent_span_id=parent_span_id,
        trace_id=impl_trace_id,
        group_id=impl_group_id,
        input=impl_input,
        output=impl_output,
        metadata=impl_metadata,
        span_type=span_type,
    )

    if is_disabled():
        return noop_span

    if impl_trace_id is None:
        log.debug(f"Attempting to create a span with no trace")
        return noop_span

    queue_manager = queue_manager or tracing_queue_manager()
    span = Span(
        name=name,
        span_id=span_id,
        parent_span_id=parent_span_id,
        trace_id=impl_trace_id,
        group_id=impl_group_id,
        input=impl_input,
        output=impl_output,
        metadata=impl_metadata,
        queue_manager=queue_manager,
        span_type=span_type,
    )
    log.debug(f"Created new span: {span.span_id}")

    return span



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["EvaluationSchemaResponse", "Field"]


class Field(BaseModel):
    """Schema information for a single field in evaluation item data"""

    data_type: str
    """JSON type: 'string', 'number', 'boolean', 'object', 'array', or 'null'"""

    field_name: str
    """The flattened JSON key path (e.g., 'metadata.category')"""

    item_count: int
    """Number of evaluation items containing this field"""

    source: Literal["data", "task_result_cache"]
    """The source of the field: 'data' or 'task_result_cache'"""

    object: Optional[Literal["field_schema"]] = None


class EvaluationSchemaResponse(BaseModel):
    """Schema information for an evaluation's item data structure"""

    evaluation_id: str
    """The ID of the evaluation"""

    fields: List[Field]
    """List of all discovered fields, ordered alphabetically by field_name"""

    total_items: int
    """Total number of evaluation items"""

    is_sampled: Optional[bool] = None
    """Whether schema was computed from a sample of items (for large evaluations)"""

    object: Optional[Literal["evaluation_schema"]] = None

    sample_size: Optional[int] = None
    """Number of items sampled for schema inference, if applicable"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

__all__ = ["EvaluationRetrieveParams"]


class EvaluationRetrieveParams(TypedDict, total=False):
    include_archived: bool

    views: List[Literal["tasks"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from .file import File
from .._models import BaseModel

__all__ = ["FileImportFromCloudResponse", "Result", "ResultFile", "ResultFile_FailedFile"]


class ResultFile_FailedFile(BaseModel):
    """
    Minimal file representation for failed uploads containing only essential information.
    """

    filename: str
    """The original filename from the request"""

    mime_type: str
    """The original MIME type from the request"""

    object: Optional[Literal["failed_file"]] = None


ResultFile: TypeAlias = Union[File, ResultFile_FailedFile]


class Result(BaseModel):
    file: ResultFile
    """The file object (full File for success, minimal \\__FailedFile for failures)"""

    status: Literal["SUCCESS", "FAILED_FILE_DOES_NOT_EXIST", "FAILED_INVALID_PERMISSIONS", "FAILED_UNKNOWN_ERROR"]
    """The status of the upload attempt"""


class FileImportFromCloudResponse(BaseModel):
    results: List[Result]
    """Results for each file import attempt"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from .._types import SequenceNotStr

__all__ = ["EvaluationDashboardUpdateParams"]


class EvaluationDashboardUpdateParams(TypedDict, total=False):
    description: str
    """Dashboard description"""

    name: str
    """Dashboard name"""

    tags: SequenceNotStr[str]
    """The tags associated with the entity"""

    widget_order: SequenceNotStr[str]
    """Ordered array of widget IDs (for reordering widgets)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["DatasetCreateParams"]


class DatasetCreateParams(TypedDict, total=False):
    data: Required[Iterable[Dict[str, object]]]
    """Items to be included in the dataset"""

    name: Required[str]

    description: str

    files: Iterable[Dict[str, str]]
    """Files to be associated to the dataset"""

    tags: SequenceNotStr[str]
    """The tags associated with the entity"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .shared.identity import Identity

__all__ = ["EvaluationItem", "TaskErrors"]


class TaskErrors(BaseModel):
    message: str
    """Error message"""

    type: str
    """Error type/category"""


class EvaluationItem(BaseModel):
    id: str

    created_at: datetime

    created_by: Identity
    """The identity that created the entity."""

    data: Dict[str, object]

    evaluation_id: str

    archived_at: Optional[datetime] = None

    dataset_item_id: Optional[str] = None

    dataset_item_version_num: Optional[int] = None

    files: Optional[Dict[str, str]] = None

    object: Optional[Literal["evaluation.item"]] = None

    task_errors: Optional[Dict[str, TaskErrors]] = None
    """Map of task alias to error info."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import TypedDict

__all__ = ["CredentialUpdateParams"]


class CredentialUpdateParams(TypedDict, total=False):
    credential_metadata: Dict[str, object]
    """Optional unencrypted credential_metadata"""

    description: str
    """Optional description"""

    name: str
    """User-friendly name for the credential"""

    payload: str
    """The credential payload to be encrypted"""

    type: str
    """Type of credential: key or json"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["VectorStoreDeleteResponse"]


class VectorStoreDeleteResponse(BaseModel):
    """Response for delete operation."""

    deleted_count: int
    """Number of documents deleted"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import builtins
from typing import TYPE_CHECKING, Dict, List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "ChatCompletionChunk",
    "Choice",
    "ChoiceDelta",
    "ChoiceDeltaFunctionCall",
    "ChoiceDeltaToolCall",
    "ChoiceDeltaToolCallFunction",
    "ChoiceLogprobs",
    "ChoiceLogprobsContent",
    "ChoiceLogprobsContentTopLogprob",
    "ChoiceLogprobsRefusal",
    "ChoiceLogprobsRefusalTopLogprob",
    "Usage",
    "UsageCompletionTokensDetails",
    "UsagePromptTokensDetails",
]


class ChoiceDeltaFunctionCall(BaseModel):
    arguments: Optional[str] = None

    name: Optional[str] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class ChoiceDeltaToolCallFunction(BaseModel):
    arguments: Optional[str] = None

    name: Optional[str] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class ChoiceDeltaToolCall(BaseModel):
    index: int

    id: Optional[str] = None

    function: Optional[ChoiceDeltaToolCallFunction] = None

    type: Optional[Literal["function"]] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class ChoiceDelta(BaseModel):
    content: Optional[str] = None

    function_call: Optional[ChoiceDeltaFunctionCall] = None

    refusal: Optional[str] = None

    role: Optional[Literal["developer", "system", "user", "assistant", "tool"]] = None

    tool_calls: Optional[List[ChoiceDeltaToolCall]] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class ChoiceLogprobsContentTopLogprob(BaseModel):
    token: str

    logprob: float

    bytes: Optional[List[int]] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class ChoiceLogprobsContent(BaseModel):
    token: str

    logprob: float

    top_logprobs: List[ChoiceLogprobsContentTopLogprob]

    bytes: Optional[List[int]] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class ChoiceLogprobsRefusalTopLogprob(BaseModel):
    token: str

    logprob: float

    bytes: Optional[List[int]] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class ChoiceLogprobsRefusal(BaseModel):
    token: str

    logprob: float

    top_logprobs: List[ChoiceLogprobsRefusalTopLogprob]

    bytes: Optional[List[int]] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class ChoiceLogprobs(BaseModel):
    content: Optional[List[ChoiceLogprobsContent]] = None

    refusal: Optional[List[ChoiceLogprobsRefusal]] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class Choice(BaseModel):
    delta: ChoiceDelta

    index: int

    finish_reason: Optional[Literal["stop", "length", "tool_calls", "content_filter", "function_call"]] = None

    logprobs: Optional[ChoiceLogprobs] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class UsageCompletionTokensDetails(BaseModel):
    accepted_prediction_tokens: Optional[int] = None

    audio_tokens: Optional[int] = None

    reasoning_tokens: Optional[int] = None

    rejected_prediction_tokens: Optional[int] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class UsagePromptTokensDetails(BaseModel):
    audio_tokens: Optional[int] = None

    cached_tokens: Optional[int] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class Usage(BaseModel):
    completion_tokens: int

    prompt_tokens: int

    total_tokens: int

    completion_tokens_details: Optional[UsageCompletionTokensDetails] = None

    prompt_tokens_details: Optional[UsagePromptTokensDetails] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class ChatCompletionChunk(BaseModel):
    id: str

    choices: List[Choice]

    created: int

    model: str

    object: Optional[Literal["chat.completion.chunk"]] = None

    service_tier: Optional[Literal["auto", "default", "flex", "scale", "priority"]] = None

    system_fingerprint: Optional[str] = None

    usage: Optional[Usage] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, builtins.object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> builtins.object: ...
    else:
        __pydantic_extra__: Dict[str, builtins.object]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from scale_gp_beta import SGPClient, AsyncSGPClient
from scale_gp_beta.types import (
    Span,
    SpanBatchResponse,
    SpanUpsertBatchResponse,
)
from scale_gp_beta._utils import parse_datetime
from scale_gp_beta.pagination import SyncCursorPage, AsyncCursorPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSpans:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        span = client.spans.create(
            name="name",
            start_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
            trace_id="trace_id",
        )
        assert_matches_type(Span, span, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: SGPClient) -> None:
        span = client.spans.create(
            name="name",
            start_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
            trace_id="trace_id",
            id="id",
            application_interaction_id="application_interaction_id",
            application_variant_id="application_variant_id",
            end_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
            group_id="group_id",
            input={"foo": "bar"},
            metadata={"foo": "bar"},
            output={"foo": "bar"},
            parent_id="parent_id",
            status="SUCCESS",
            type="TEXT_INPUT",
        )
        assert_matches_type(Span, span, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.spans.with_raw_response.create(
            name="name",
            start_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
            trace_id="trace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        span = response.parse()
        assert_matches_type(Span, span, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.spans.with_streaming_response.create(
            name="name",
            start_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
            trace_id="trace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            span = response.parse()
            assert_matches_type(Span, span, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        span = client.spans.retrieve(
            "span_id",
        )
        assert_matches_type(Span, span, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.spans.with_raw_response.retrieve(
            "span_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        span = response.parse()
        assert_matches_type(Span, span, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.spans.with_streaming_response.retrieve(
            "span_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            span = response.parse()
            assert_matches_type(Span, span, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `span_id` but received ''"):
            client.spans.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: SGPClient) -> None:
        span = client.spans.update(
            span_id="span_id",
        )
        assert_matches_type(Span, span, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: SGPClient) -> None:
        span = client.spans.update(
            span_id="span_id",
            end_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
            metadata={"foo": "bar"},
            name="name",
            output={"foo": "bar"},
            status="SUCCESS",
        )
        assert_matches_type(Span, span, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: SGPClient) -> None:
        response = client.spans.with_raw_response.update(
            span_id="span_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        span = response.parse()
        assert_matches_type(Span, span, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: SGPClient) -> None:
        with client.spans.with_streaming_response.update(
            span_id="span_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            span = response.parse()
            assert_matches_type(Span, span, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `span_id` but received ''"):
            client.spans.with_raw_response.update(
                span_id="",
            )

    @parametrize
    def test_method_batch(self, client: SGPClient) -> None:
        span = client.spans.batch(
            items=[
                {
                    "name": "name",
                    "start_timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "trace_id": "trace_id",
                }
            ],
        )
        assert_matches_type(SpanBatchResponse, span, path=["response"])

    @parametrize
    def test_raw_response_batch(self, client: SGPClient) -> None:
        response = client.spans.with_raw_response.batch(
            items=[
                {
                    "name": "name",
                    "start_timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "trace_id": "trace_id",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        span = response.parse()
        assert_matches_type(SpanBatchResponse, span, path=["response"])

    @parametrize
    def test_streaming_response_batch(self, client: SGPClient) -> None:
        with client.spans.with_streaming_response.batch(
            items=[
                {
                    "name": "name",
                    "start_timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "trace_id": "trace_id",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            span = response.parse()
            assert_matches_type(SpanBatchResponse, span, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_search(self, client: SGPClient) -> None:
        span = client.spans.search()
        assert_matches_type(SyncCursorPage[Span], span, path=["response"])

    @parametrize
    def test_method_search_with_all_params(self, client: SGPClient) -> None:
        span = client.spans.search(
            ending_before="ending_before",
            from_ts=parse_datetime("2019-12-27T18:11:19.117Z"),
            limit=1,
            sort_by="sort_by",
            sort_order="asc",
            starting_after="starting_after",
            to_ts=parse_datetime("2019-12-27T18:11:19.117Z"),
            acp_types=["string"],
            agentex_agent_ids=["string"],
            agentex_agent_names=["string"],
            application_variant_ids=["string"],
            assessment_types=["string"],
            excluded_span_ids=["string"],
            excluded_trace_ids=["string"],
            extra_metadata={"foo": "bar"},
            group_id="group_id",
            max_duration_ms=0,
            min_duration_ms=0,
            names=["string"],
            parents_only=True,
            search_texts=["string"],
            span_ids=["string"],
            statuses=["SUCCESS"],
            trace_ids=["string"],
            types=["TEXT_INPUT"],
        )
        assert_matches_type(SyncCursorPage[Span], span, path=["response"])

    @parametrize
    def test_raw_response_search(self, client: SGPClient) -> None:
        response = client.spans.with_raw_response.search()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        span = response.parse()
        assert_matches_type(SyncCursorPage[Span], span, path=["response"])

    @parametrize
    def test_streaming_response_search(self, client: SGPClient) -> None:
        with client.spans.with_streaming_response.search() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            span = response.parse()
            assert_matches_type(SyncCursorPage[Span], span, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_upsert_batch(self, client: SGPClient) -> None:
        span = client.spans.upsert_batch(
            items=[
                {
                    "name": "name",
                    "start_timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "trace_id": "trace_id",
                }
            ],
        )
        assert_matches_type(SpanUpsertBatchResponse, span, path=["response"])

    @parametrize
    def test_raw_response_upsert_batch(self, client: SGPClient) -> None:
        response = client.spans.with_raw_response.upsert_batch(
            items=[
                {
                    "name": "name",
                    "start_timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "trace_id": "trace_id",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        span = response.parse()
        assert_matches_type(SpanUpsertBatchResponse, span, path=["response"])

    @parametrize
    def test_streaming_response_upsert_batch(self, client: SGPClient) -> None:
        with client.spans.with_streaming_response.upsert_batch(
            items=[
                {
                    "name": "name",
                    "start_timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "trace_id": "trace_id",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            span = response.parse()
            assert_matches_type(SpanUpsertBatchResponse, span, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSpans:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        span = await async_client.spans.create(
            name="name",
            start_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
            trace_id="trace_id",
        )
        assert_matches_type(Span, span, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        span = await async_client.spans.create(
            name="name",
            start_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
            trace_id="trace_id",
            id="id",
            application_interaction_id="application_interaction_id",
            application_variant_id="application_variant_id",
            end_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
            group_id="group_id",
            input={"foo": "bar"},
            metadata={"foo": "bar"},
            output={"foo": "bar"},
            parent_id="parent_id",
            status="SUCCESS",
            type="TEXT_INPUT",
        )
        assert_matches_type(Span, span, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.spans.with_raw_response.create(
            name="name",
            start_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
            trace_id="trace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        span = await response.parse()
        assert_matches_type(Span, span, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.spans.with_streaming_response.create(
            name="name",
            start_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
            trace_id="trace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            span = await response.parse()
            assert_matches_type(Span, span, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        span = await async_client.spans.retrieve(
            "span_id",
        )
        assert_matches_type(Span, span, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.spans.with_raw_response.retrieve(
            "span_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        span = await response.parse()
        assert_matches_type(Span, span, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.spans.with_streaming_response.retrieve(
            "span_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            span = await response.parse()
            assert_matches_type(Span, span, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `span_id` but received ''"):
            await async_client.spans.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncSGPClient) -> None:
        span = await async_client.spans.update(
            span_id="span_id",
        )
        assert_matches_type(Span, span, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncSGPClient) -> None:
        span = await async_client.spans.update(
            span_id="span_id",
            end_timestamp=parse_datetime("2019-12-27T18:11:19.117Z"),
            metadata={"foo": "bar"},
            name="name",
            output={"foo": "bar"},
            status="SUCCESS",
        )
        assert_matches_type(Span, span, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.spans.with_raw_response.update(
            span_id="span_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        span = await response.parse()
        assert_matches_type(Span, span, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncSGPClient) -> None:
        async with async_client.spans.with_streaming_response.update(
            span_id="span_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            span = await response.parse()
            assert_matches_type(Span, span, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `span_id` but received ''"):
            await async_client.spans.with_raw_response.update(
                span_id="",
            )

    @parametrize
    async def test_method_batch(self, async_client: AsyncSGPClient) -> None:
        span = await async_client.spans.batch(
            items=[
                {
                    "name": "name",
                    "start_timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "trace_id": "trace_id",
                }
            ],
        )
        assert_matches_type(SpanBatchResponse, span, path=["response"])

    @parametrize
    async def test_raw_response_batch(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.spans.with_raw_response.batch(
            items=[
                {
                    "name": "name",
                    "start_timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "trace_id": "trace_id",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        span = await response.parse()
        assert_matches_type(SpanBatchResponse, span, path=["response"])

    @parametrize
    async def test_streaming_response_batch(self, async_client: AsyncSGPClient) -> None:
        async with async_client.spans.with_streaming_response.batch(
            items=[
                {
                    "name": "name",
                    "start_timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "trace_id": "trace_id",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            span = await response.parse()
            assert_matches_type(SpanBatchResponse, span, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_search(self, async_client: AsyncSGPClient) -> None:
        span = await async_client.spans.search()
        assert_matches_type(AsyncCursorPage[Span], span, path=["response"])

    @parametrize
    async def test_method_search_with_all_params(self, async_client: AsyncSGPClient) -> None:
        span = await async_client.spans.search(
            ending_before="ending_before",
            from_ts=parse_datetime("2019-12-27T18:11:19.117Z"),
            limit=1,
            sort_by="sort_by",
            sort_order="asc",
            starting_after="starting_after",
            to_ts=parse_datetime("2019-12-27T18:11:19.117Z"),
            acp_types=["string"],
            agentex_agent_ids=["string"],
            agentex_agent_names=["string"],
            application_variant_ids=["string"],
            assessment_types=["string"],
            excluded_span_ids=["string"],
            excluded_trace_ids=["string"],
            extra_metadata={"foo": "bar"},
            group_id="group_id",
            max_duration_ms=0,
            min_duration_ms=0,
            names=["string"],
            parents_only=True,
            search_texts=["string"],
            span_ids=["string"],
            statuses=["SUCCESS"],
            trace_ids=["string"],
            types=["TEXT_INPUT"],
        )
        assert_matches_type(AsyncCursorPage[Span], span, path=["response"])

    @parametrize
    async def test_raw_response_search(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.spans.with_raw_response.search()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        span = await response.parse()
        assert_matches_type(AsyncCursorPage[Span], span, path=["response"])

    @parametrize
    async def test_streaming_response_search(self, async_client: AsyncSGPClient) -> None:
        async with async_client.spans.with_streaming_response.search() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            span = await response.parse()
            assert_matches_type(AsyncCursorPage[Span], span, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_upsert_batch(self, async_client: AsyncSGPClient) -> None:
        span = await async_client.spans.upsert_batch(
            items=[
                {
                    "name": "name",
                    "start_timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "trace_id": "trace_id",
                }
            ],
        )
        assert_matches_type(SpanUpsertBatchResponse, span, path=["response"])

    @parametrize
    async def test_raw_response_upsert_batch(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.spans.with_raw_response.upsert_batch(
            items=[
                {
                    "name": "name",
                    "start_timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "trace_id": "trace_id",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        span = await response.parse()
        assert_matches_type(SpanUpsertBatchResponse, span, path=["response"])

    @parametrize
    async def test_streaming_response_upsert_batch(self, async_client: AsyncSGPClient) -> None:
        async with async_client.spans.with_streaming_response.upsert_batch(
            items=[
                {
                    "name": "name",
                    "start_timestamp": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "trace_id": "trace_id",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            span = await response.parse()
            assert_matches_type(SpanUpsertBatchResponse, span, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Literal

import httpx

from ..types import (
    vector_store_list_params,
    vector_store_count_params,
    vector_store_query_params,
    vector_store_create_params,
    vector_store_delete_params,
    vector_store_upsert_params,
    vector_store_vectors_params,
    vector_store_configure_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorPage, AsyncCursorPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.vector_store import VectorStore
from ..types.vector_store_drop_response import VectorStoreDropResponse
from ..types.vector_store_count_response import VectorStoreCountResponse
from ..types.vector_store_query_response import VectorStoreQueryResponse
from ..types.vector_store_delete_response import VectorStoreDeleteResponse
from ..types.vector_store_upsert_response import VectorStoreUpsertResponse
from ..types.vector_store_vectors_response import VectorStoreVectorsResponse

__all__ = ["VectorStoresResource", "AsyncVectorStoresResource"]


class VectorStoresResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VectorStoresResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#accessing-raw-response-data-eg-headers
        """
        return VectorStoresResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VectorStoresResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#with_streaming_response
        """
        return VectorStoresResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        embedding_model: Literal[
            "sentence-transformers/all-MiniLM-L12-v2",
            "sentence-transformers/all-mpnet-base-v2",
            "sentence-transformers/multi-qa-distilbert-cos-v1",
            "sentence-transformers/paraphrase-multilingual-mpnet-base-v2",
            "openai/text-embedding-ada-002",
            "openai/text-embedding-3-small",
            "openai/text-embedding-3-large",
            "embed-english-v3.0",
            "embed-english-light-v3.0",
            "embed-multilingual-v3.0",
        ],
        name: str,
        indexed_metadata_fields: Dict[str, Literal["string", "number", "boolean", "object", "list"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VectorStore:
        """
        Create a new vector store.

        The vector store will be scoped to the authenticated account. The name must be
        unique within the account.

        Args: request: Vector store creation parameters including name, dimensions, and
        optional model vector_store_use_case: Injected vector store use case

        Returns: The created vector store details

        Args:
          embedding_model: The embedding model to use

          name: A unique name for the vector store within the account

          indexed_metadata_fields: Dictionary mapping metadata field names to their types for efficient filtering.
              Only STRING, NUMBER, and BOOLEAN types can be indexed.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v5/vector-stores/create",
            body=maybe_transform(
                {
                    "embedding_model": embedding_model,
                    "name": name,
                    "indexed_metadata_fields": indexed_metadata_fields,
                },
                vector_store_create_params.VectorStoreCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VectorStore,
        )

    def retrieve(
        self,
        vector_store_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VectorStore:
        """
        Get a vector store by name.

        Args: vector_store_name: The unique name of the vector store within the account
        vector_store_use_case: Injected vector store use case

        Returns: The vector store details

        Args:
          vector_store_name: The name of the vector store

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not vector_store_name:
            raise ValueError(f"Expected a non-empty value for `vector_store_name` but received {vector_store_name!r}")
        return self._get(
            f"/v5/vector-stores/{vector_store_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VectorStore,
        )

    def list(
        self,
        *,
        ending_before: str | Omit = omit,
        limit: int | Omit = omit,
        sort_by: str | Omit = omit,
        sort_order: Literal["asc", "desc"] | Omit = omit,
        starting_after: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[VectorStore]:
        """
        List all vector stores for the current account with pagination.

        Returns a paginated list of vector stores owned by the authenticated account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v5/vector-stores",
            page=SyncCursorPage[VectorStore],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "limit": limit,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                    },
                    vector_store_list_params.VectorStoreListParams,
                ),
            ),
            model=VectorStore,
        )

    def delete(
        self,
        vector_store_name: str,
        *,
        filter: Dict[str, object] | Omit = omit,
        ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VectorStoreDeleteResponse:
        """
        Delete documents from a vector store by IDs or filter.

        Args: request: Either IDs or filter criteria for deletion vector_store_name: The
        unique name of the vector store vector_store_use_case: Injected vector store use
        case

        Returns: Number of documents deleted

        Args:
          vector_store_name: The name of the vector store

          filter: Metadata filter expression for deletion

          ids: Array of document IDs to delete

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not vector_store_name:
            raise ValueError(f"Expected a non-empty value for `vector_store_name` but received {vector_store_name!r}")
        return self._post(
            f"/v5/vector-stores/{vector_store_name}/delete",
            body=maybe_transform(
                {
                    "filter": filter,
                    "ids": ids,
                },
                vector_store_delete_params.VectorStoreDeleteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VectorStoreDeleteResponse,
        )

    def configure(
        self,
        vector_store_name: str,
        *,
        indexed_metadata_fields: Dict[str, Literal["string", "number", "boolean", "object", "list"]],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VectorStore:
        """
        Configure the settings of a vector store.

        Currently only supports updating indexed_metadata_fields. The name,
        embedding_dimensions, and embedding_model are immutable after creation.

        Args: request: Configuration update with indexed_metadata_fields
        vector_store_name: The unique name of the vector store within the account
        vector_store_use_case: Injected vector store use case

        Returns: The updated vector store details

        Args:
          vector_store_name: The name of the vector store

          indexed_metadata_fields: Dictionary mapping metadata field names to their types. Only STRING, NUMBER, and
              BOOLEAN types can be indexed.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not vector_store_name:
            raise ValueError(f"Expected a non-empty value for `vector_store_name` but received {vector_store_name!r}")
        return self._post(
            f"/v5/vector-stores/{vector_store_name}/configure",
            body=maybe_transform(
                {"indexed_metadata_fields": indexed_metadata_fields},
                vector_store_configure_params.VectorStoreConfigureParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VectorStore,
        )

    def count(
        self,
        vector_store_name: str,
        *,
        filter: Dict[str, object] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VectorStoreCountResponse:
        """
        Count documents in a vector store, optionally filtered by metadata.

        Args: vector_store_name: The unique name of the vector store request: Optional
        filter criteria (empty body counts all documents) vector_store_use_case:
        Injected vector store use case

        Returns: The count of documents matching the criteria

        Args:
          vector_store_name: The name of the vector store

          filter: Metadata filter expression

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not vector_store_name:
            raise ValueError(f"Expected a non-empty value for `vector_store_name` but received {vector_store_name!r}")
        return self._post(
            f"/v5/vector-stores/{vector_store_name}/count",
            body=maybe_transform({"filter": filter}, vector_store_count_params.VectorStoreCountParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VectorStoreCountResponse,
        )

    def drop(
        self,
        vector_store_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VectorStoreDropResponse:
        """
        Delete (drop) a vector store by name.

        This is a hard delete operation that permanently removes the vector store.

        Args: vector_store_name: The unique name of the vector store within the account
        vector_store_use_case: Injected vector store use case

        Returns: Deletion confirmation with the vector store ID

        Args:
          vector_store_name: The name of the vector store

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not vector_store_name:
            raise ValueError(f"Expected a non-empty value for `vector_store_name` but received {vector_store_name!r}")
        return self._post(
            f"/v5/vector-stores/{vector_store_name}/drop",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VectorStoreDropResponse,
        )

    def query(
        self,
        vector_store_name: str,
        *,
        text: str,
        filter: Dict[str, object] | Omit = omit,
        include_vectors: bool | Omit = omit,
        query_type: Literal["semantic", "lexical", "hybrid"] | Omit = omit,
        rerank: bool | Omit = omit,
        rerank_model: str | Omit = omit,
        rerank_top_n: int | Omit = omit,
        top_k: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VectorStoreQueryResponse:
        """
        Query documents by similarity search.

        Supports semantic (vector), lexical (text), or hybrid search modes.

        Args: request: Query parameters including text, filters, and reranking options
        vector_store_name: The unique name of the vector store vector_store_use_case:
        Injected vector store use case

        Returns: Matching documents with similarity scores and query metadata

        Args:
          vector_store_name: The name of the vector store

          text: Text query for automatic embedding (required)

          filter: Metadata filter expression

          include_vectors: Include embedding vectors in response

          query_type: Query type: semantic, lexical, or hybrid

          rerank: Enable reranking of search results

          rerank_model: Reranking model to use (uses system default if not specified)

          rerank_top_n: Number of results after reranking (defaults to top_k)

          top_k: Number of search results to return

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not vector_store_name:
            raise ValueError(f"Expected a non-empty value for `vector_store_name` but received {vector_store_name!r}")
        return self._post(
            f"/v5/vector-stores/{vector_store_name}/query",
            body=maybe_transform(
                {
                    "text": text,
                    "filter": filter,
                    "include_vectors": include_vectors,
                    "query_type": query_type,
                    "rerank": rerank,
                    "rerank_model": rerank_model,
                    "rerank_top_n": rerank_top_n,
                    "top_k": top_k,
                },
                vector_store_query_params.VectorStoreQueryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VectorStoreQueryResponse,
        )

    def upsert(
        self,
        vector_store_name: str,
        *,
        vectors: Iterable[vector_store_upsert_params.Vector],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VectorStoreUpsertResponse:
        """
        Insert or update documents in a vector store.

        Existing documents (by ID) will be updated, new documents will be inserted.

        Args: request: Array of documents to upsert vector_store_name: The unique name
        of the vector store vector_store_use_case: Injected vector store use case

        Returns: Batch operation results with success/failure counts

        Args:
          vector_store_name: The name of the vector store

          vectors: Array of documents to upsert

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not vector_store_name:
            raise ValueError(f"Expected a non-empty value for `vector_store_name` but received {vector_store_name!r}")
        return self._post(
            f"/v5/vector-stores/{vector_store_name}/upsert",
            body=maybe_transform({"vectors": vectors}, vector_store_upsert_params.VectorStoreUpsertParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VectorStoreUpsertResponse,
        )

    def vectors(
        self,
        vector_store_name: str,
        *,
        cursor: str | Omit = omit,
        filter: str | Omit = omit,
        include_vectors: bool | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VectorStoreVectorsResponse:
        """
        List documents in a vector store with pagination.

        Args: vector_store_name: The unique name of the vector store limit: Maximum
        number of documents per page (max 100 with vectors, 1000 without) cursor:
        Pagination cursor from previous response filter_param: Optional metadata filter
        (JSON string) include_vectors: Whether to include embedding vectors
        vector_store_use_case: Injected vector store use case

        Returns: Paginated list of documents

        Args:
          vector_store_name: The name of the vector store

          cursor: Pagination cursor

          filter: Metadata filter expression (JSON)

          include_vectors: Include embedding vectors

          limit: Vectors per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not vector_store_name:
            raise ValueError(f"Expected a non-empty value for `vector_store_name` but received {vector_store_name!r}")
        return self._get(
            f"/v5/vector-stores/{vector_store_name}/vectors",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "filter": filter,
                        "include_vectors": include_vectors,
                        "limit": limit,
                    },
                    vector_store_vectors_params.VectorStoreVectorsParams,
                ),
            ),
            cast_to=VectorStoreVectorsResponse,
        )


class AsyncVectorStoresResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVectorStoresResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#accessing-raw-response-data-eg-headers
        """
        return AsyncVectorStoresResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVectorStoresResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#with_streaming_response
        """
        return AsyncVectorStoresResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        embedding_model: Literal[
            "sentence-transformers/all-MiniLM-L12-v2",
            "sentence-transformers/all-mpnet-base-v2",
            "sentence-transformers/multi-qa-distilbert-cos-v1",
            "sentence-transformers/paraphrase-multilingual-mpnet-base-v2",
            "openai/text-embedding-ada-002",
            "openai/text-embedding-3-small",
            "openai/text-embedding-3-large",
            "embed-english-v3.0",
            "embed-english-light-v3.0",
            "embed-multilingual-v3.0",
        ],
        name: str,
        indexed_metadata_fields: Dict[str, Literal["string", "number", "boolean", "object", "list"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VectorStore:
        """
        Create a new vector store.

        The vector store will be scoped to the authenticated account. The name must be
        unique within the account.

        Args: request: Vector store creation parameters including name, dimensions, and
        optional model vector_store_use_case: Injected vector store use case

        Returns: The created vector store details

        Args:
          embedding_model: The embedding model to use

          name: A unique name for the vector store within the account

          indexed_metadata_fields: Dictionary mapping metadata field names to their types for efficient filtering.
              Only STRING, NUMBER, and BOOLEAN types can be indexed.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v5/vector-stores/create",
            body=await async_maybe_transform(
                {
                    "embedding_model": embedding_model,
                    "name": name,
                    "indexed_metadata_fields": indexed_metadata_fields,
                },
                vector_store_create_params.VectorStoreCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VectorStore,
        )

    async def retrieve(
        self,
        vector_store_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VectorStore:
        """
        Get a vector store by name.

        Args: vector_store_name: The unique name of the vector store within the account
        vector_store_use_case: Injected vector store use case

        Returns: The vector store details

        Args:
          vector_store_name: The name of the vector store

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not vector_store_name:
            raise ValueError(f"Expected a non-empty value for `vector_store_name` but received {vector_store_name!r}")
        return await self._get(
            f"/v5/vector-stores/{vector_store_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VectorStore,
        )

    def list(
        self,
        *,
        ending_before: str | Omit = omit,
        limit: int | Omit = omit,
        sort_by: str | Omit = omit,
        sort_order: Literal["asc", "desc"] | Omit = omit,
        starting_after: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[VectorStore, AsyncCursorPage[VectorStore]]:
        """
        List all vector stores for the current account with pagination.

        Returns a paginated list of vector stores owned by the authenticated account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v5/vector-stores",
            page=AsyncCursorPage[VectorStore],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "limit": limit,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                    },
                    vector_store_list_params.VectorStoreListParams,
                ),
            ),
            model=VectorStore,
        )

    async def delete(
        self,
        vector_store_name: str,
        *,
        filter: Dict[str, object] | Omit = omit,
        ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VectorStoreDeleteResponse:
        """
        Delete documents from a vector store by IDs or filter.

        Args: request: Either IDs or filter criteria for deletion vector_store_name: The
        unique name of the vector store vector_store_use_case: Injected vector store use
        case

        Returns: Number of documents deleted

        Args:
          vector_store_name: The name of the vector store

          filter: Metadata filter expression for deletion

          ids: Array of document IDs to delete

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not vector_store_name:
            raise ValueError(f"Expected a non-empty value for `vector_store_name` but received {vector_store_name!r}")
        return await self._post(
            f"/v5/vector-stores/{vector_store_name}/delete",
            body=await async_maybe_transform(
                {
                    "filter": filter,
                    "ids": ids,
                },
                vector_store_delete_params.VectorStoreDeleteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VectorStoreDeleteResponse,
        )

    async def configure(
        self,
        vector_store_name: str,
        *,
        indexed_metadata_fields: Dict[str, Literal["string", "number", "boolean", "object", "list"]],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VectorStore:
        """
        Configure the settings of a vector store.

        Currently only supports updating indexed_metadata_fields. The name,
        embedding_dimensions, and embedding_model are immutable after creation.

        Args: request: Configuration update with indexed_metadata_fields
        vector_store_name: The unique name of the vector store within the account
        vector_store_use_case: Injected vector store use case

        Returns: The updated vector store details

        Args:
          vector_store_name: The name of the vector store

          indexed_metadata_fields: Dictionary mapping metadata field names to their types. Only STRING, NUMBER, and
              BOOLEAN types can be indexed.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not vector_store_name:
            raise ValueError(f"Expected a non-empty value for `vector_store_name` but received {vector_store_name!r}")
        return await self._post(
            f"/v5/vector-stores/{vector_store_name}/configure",
            body=await async_maybe_transform(
                {"indexed_metadata_fields": indexed_metadata_fields},
                vector_store_configure_params.VectorStoreConfigureParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VectorStore,
        )

    async def count(
        self,
        vector_store_name: str,
        *,
        filter: Dict[str, object] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VectorStoreCountResponse:
        """
        Count documents in a vector store, optionally filtered by metadata.

        Args: vector_store_name: The unique name of the vector store request: Optional
        filter criteria (empty body counts all documents) vector_store_use_case:
        Injected vector store use case

        Returns: The count of documents matching the criteria

        Args:
          vector_store_name: The name of the vector store

          filter: Metadata filter expression

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not vector_store_name:
            raise ValueError(f"Expected a non-empty value for `vector_store_name` but received {vector_store_name!r}")
        return await self._post(
            f"/v5/vector-stores/{vector_store_name}/count",
            body=await async_maybe_transform({"filter": filter}, vector_store_count_params.VectorStoreCountParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VectorStoreCountResponse,
        )

    async def drop(
        self,
        vector_store_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VectorStoreDropResponse:
        """
        Delete (drop) a vector store by name.

        This is a hard delete operation that permanently removes the vector store.

        Args: vector_store_name: The unique name of the vector store within the account
        vector_store_use_case: Injected vector store use case

        Returns: Deletion confirmation with the vector store ID

        Args:
          vector_store_name: The name of the vector store

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not vector_store_name:
            raise ValueError(f"Expected a non-empty value for `vector_store_name` but received {vector_store_name!r}")
        return await self._post(
            f"/v5/vector-stores/{vector_store_name}/drop",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VectorStoreDropResponse,
        )

    async def query(
        self,
        vector_store_name: str,
        *,
        text: str,
        filter: Dict[str, object] | Omit = omit,
        include_vectors: bool | Omit = omit,
        query_type: Literal["semantic", "lexical", "hybrid"] | Omit = omit,
        rerank: bool | Omit = omit,
        rerank_model: str | Omit = omit,
        rerank_top_n: int | Omit = omit,
        top_k: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VectorStoreQueryResponse:
        """
        Query documents by similarity search.

        Supports semantic (vector), lexical (text), or hybrid search modes.

        Args: request: Query parameters including text, filters, and reranking options
        vector_store_name: The unique name of the vector store vector_store_use_case:
        Injected vector store use case

        Returns: Matching documents with similarity scores and query metadata

        Args:
          vector_store_name: The name of the vector store

          text: Text query for automatic embedding (required)

          filter: Metadata filter expression

          include_vectors: Include embedding vectors in response

          query_type: Query type: semantic, lexical, or hybrid

          rerank: Enable reranking of search results

          rerank_model: Reranking model to use (uses system default if not specified)

          rerank_top_n: Number of results after reranking (defaults to top_k)

          top_k: Number of search results to return

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not vector_store_name:
            raise ValueError(f"Expected a non-empty value for `vector_store_name` but received {vector_store_name!r}")
        return await self._post(
            f"/v5/vector-stores/{vector_store_name}/query",
            body=await async_maybe_transform(
                {
                    "text": text,
                    "filter": filter,
                    "include_vectors": include_vectors,
                    "query_type": query_type,
                    "rerank": rerank,
                    "rerank_model": rerank_model,
                    "rerank_top_n": rerank_top_n,
                    "top_k": top_k,
                },
                vector_store_query_params.VectorStoreQueryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VectorStoreQueryResponse,
        )

    async def upsert(
        self,
        vector_store_name: str,
        *,
        vectors: Iterable[vector_store_upsert_params.Vector],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VectorStoreUpsertResponse:
        """
        Insert or update documents in a vector store.

        Existing documents (by ID) will be updated, new documents will be inserted.

        Args: request: Array of documents to upsert vector_store_name: The unique name
        of the vector store vector_store_use_case: Injected vector store use case

        Returns: Batch operation results with success/failure counts

        Args:
          vector_store_name: The name of the vector store

          vectors: Array of documents to upsert

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not vector_store_name:
            raise ValueError(f"Expected a non-empty value for `vector_store_name` but received {vector_store_name!r}")
        return await self._post(
            f"/v5/vector-stores/{vector_store_name}/upsert",
            body=await async_maybe_transform({"vectors": vectors}, vector_store_upsert_params.VectorStoreUpsertParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VectorStoreUpsertResponse,
        )

    async def vectors(
        self,
        vector_store_name: str,
        *,
        cursor: str | Omit = omit,
        filter: str | Omit = omit,
        include_vectors: bool | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VectorStoreVectorsResponse:
        """
        List documents in a vector store with pagination.

        Args: vector_store_name: The unique name of the vector store limit: Maximum
        number of documents per page (max 100 with vectors, 1000 without) cursor:
        Pagination cursor from previous response filter_param: Optional metadata filter
        (JSON string) include_vectors: Whether to include embedding vectors
        vector_store_use_case: Injected vector store use case

        Returns: Paginated list of documents

        Args:
          vector_store_name: The name of the vector store

          cursor: Pagination cursor

          filter: Metadata filter expression (JSON)

          include_vectors: Include embedding vectors

          limit: Vectors per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not vector_store_name:
            raise ValueError(f"Expected a non-empty value for `vector_store_name` but received {vector_store_name!r}")
        return await self._get(
            f"/v5/vector-stores/{vector_store_name}/vectors",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "cursor": cursor,
                        "filter": filter,
                        "include_vectors": include_vectors,
                        "limit": limit,
                    },
                    vector_store_vectors_params.VectorStoreVectorsParams,
                ),
            ),
            cast_to=VectorStoreVectorsResponse,
        )


class VectorStoresResourceWithRawResponse:
    def __init__(self, vector_stores: VectorStoresResource) -> None:
        self._vector_stores = vector_stores

        self.create = to_raw_response_wrapper(
            vector_stores.create,
        )
        self.retrieve = to_raw_response_wrapper(
            vector_stores.retrieve,
        )
        self.list = to_raw_response_wrapper(
            vector_stores.list,
        )
        self.delete = to_raw_response_wrapper(
            vector_stores.delete,
        )
        self.configure = to_raw_response_wrapper(
            vector_stores.configure,
        )
        self.count = to_raw_response_wrapper(
            vector_stores.count,
        )
        self.drop = to_raw_response_wrapper(
            vector_stores.drop,
        )
        self.query = to_raw_response_wrapper(
            vector_stores.query,
        )
        self.upsert = to_raw_response_wrapper(
            vector_stores.upsert,
        )
        self.vectors = to_raw_response_wrapper(
            vector_stores.vectors,
        )


class AsyncVectorStoresResourceWithRawResponse:
    def __init__(self, vector_stores: AsyncVectorStoresResource) -> None:
        self._vector_stores = vector_stores

        self.create = async_to_raw_response_wrapper(
            vector_stores.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            vector_stores.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            vector_stores.list,
        )
        self.delete = async_to_raw_response_wrapper(
            vector_stores.delete,
        )
        self.configure = async_to_raw_response_wrapper(
            vector_stores.configure,
        )
        self.count = async_to_raw_response_wrapper(
            vector_stores.count,
        )
        self.drop = async_to_raw_response_wrapper(
            vector_stores.drop,
        )
        self.query = async_to_raw_response_wrapper(
            vector_stores.query,
        )
        self.upsert = async_to_raw_response_wrapper(
            vector_stores.upsert,
        )
        self.vectors = async_to_raw_response_wrapper(
            vector_stores.vectors,
        )


class VectorStoresResourceWithStreamingResponse:
    def __init__(self, vector_stores: VectorStoresResource) -> None:
        self._vector_stores = vector_stores

        self.create = to_streamed_response_wrapper(
            vector_stores.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            vector_stores.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            vector_stores.list,
        )
        self.delete = to_streamed_response_wrapper(
            vector_stores.delete,
        )
        self.configure = to_streamed_response_wrapper(
            vector_stores.configure,
        )
        self.count = to_streamed_response_wrapper(
            vector_stores.count,
        )
        self.drop = to_streamed_response_wrapper(
            vector_stores.drop,
        )
        self.query = to_streamed_response_wrapper(
            vector_stores.query,
        )
        self.upsert = to_streamed_response_wrapper(
            vector_stores.upsert,
        )
        self.vectors = to_streamed_response_wrapper(
            vector_stores.vectors,
        )


class AsyncVectorStoresResourceWithStreamingResponse:
    def __init__(self, vector_stores: AsyncVectorStoresResource) -> None:
        self._vector_stores = vector_stores

        self.create = async_to_streamed_response_wrapper(
            vector_stores.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            vector_stores.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            vector_stores.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            vector_stores.delete,
        )
        self.configure = async_to_streamed_response_wrapper(
            vector_stores.configure,
        )
        self.count = async_to_streamed_response_wrapper(
            vector_stores.count,
        )
        self.drop = async_to_streamed_response_wrapper(
            vector_stores.drop,
        )
        self.query = async_to_streamed_response_wrapper(
            vector_stores.query,
        )
        self.upsert = async_to_streamed_response_wrapper(
            vector_stores.upsert,
        )
        self.vectors = async_to_streamed_response_wrapper(
            vector_stores.vectors,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["Credential"]


class Credential(BaseModel):
    id: str

    created_at: datetime

    created_by_identity_type: str

    created_by_user_id: str

    credential_metadata: Dict[str, object]

    description: str

    name: str

    type: str

    updated_at: datetime

    object: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from scale_gp_beta import SGPClient, AsyncSGPClient
from scale_gp_beta.types import (
    EvaluationDashboard,
)
from scale_gp_beta.pagination import SyncCursorPage, AsyncCursorPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestEvaluationDashboards:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        evaluation_dashboard = client.evaluation_dashboards.create(
            name="x",
        )
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: SGPClient) -> None:
        evaluation_dashboard = client.evaluation_dashboards.create(
            name="x",
            description="description",
            evaluation_group_id="evaluation_group_id",
            evaluation_id="evaluation_id",
            tags=["string"],
            template_dashboard_id="template_dashboard_id",
            widget_order=["string"],
        )
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.evaluation_dashboards.with_raw_response.create(
            name="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluation_dashboard = response.parse()
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.evaluation_dashboards.with_streaming_response.create(
            name="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluation_dashboard = response.parse()
            assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        evaluation_dashboard = client.evaluation_dashboards.retrieve(
            dashboard_id="dashboard_id",
        )
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    def test_method_retrieve_with_all_params(self, client: SGPClient) -> None:
        evaluation_dashboard = client.evaluation_dashboards.retrieve(
            dashboard_id="dashboard_id",
            include_archived=True,
            views=["widgets"],
        )
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.evaluation_dashboards.with_raw_response.retrieve(
            dashboard_id="dashboard_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluation_dashboard = response.parse()
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.evaluation_dashboards.with_streaming_response.retrieve(
            dashboard_id="dashboard_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluation_dashboard = response.parse()
            assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dashboard_id` but received ''"):
            client.evaluation_dashboards.with_raw_response.retrieve(
                dashboard_id="",
            )

    @parametrize
    def test_method_update(self, client: SGPClient) -> None:
        evaluation_dashboard = client.evaluation_dashboards.update(
            dashboard_id="dashboard_id",
        )
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: SGPClient) -> None:
        evaluation_dashboard = client.evaluation_dashboards.update(
            dashboard_id="dashboard_id",
            description="description",
            name="x",
            tags=["string"],
            widget_order=["string"],
        )
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: SGPClient) -> None:
        response = client.evaluation_dashboards.with_raw_response.update(
            dashboard_id="dashboard_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluation_dashboard = response.parse()
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: SGPClient) -> None:
        with client.evaluation_dashboards.with_streaming_response.update(
            dashboard_id="dashboard_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluation_dashboard = response.parse()
            assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dashboard_id` but received ''"):
            client.evaluation_dashboards.with_raw_response.update(
                dashboard_id="",
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        evaluation_dashboard = client.evaluation_dashboards.list()
        assert_matches_type(SyncCursorPage[EvaluationDashboard], evaluation_dashboard, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        evaluation_dashboard = client.evaluation_dashboards.list(
            created_by_ids=["string"],
            ending_before="ending_before",
            evaluation_group_id="evaluation_group_id",
            evaluation_id="evaluation_id",
            include_archived=True,
            limit=1,
            search="search",
            sort_by="sort_by",
            sort_order="asc",
            starting_after="starting_after",
            tags=["string"],
        )
        assert_matches_type(SyncCursorPage[EvaluationDashboard], evaluation_dashboard, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.evaluation_dashboards.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluation_dashboard = response.parse()
        assert_matches_type(SyncCursorPage[EvaluationDashboard], evaluation_dashboard, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.evaluation_dashboards.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluation_dashboard = response.parse()
            assert_matches_type(SyncCursorPage[EvaluationDashboard], evaluation_dashboard, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: SGPClient) -> None:
        evaluation_dashboard = client.evaluation_dashboards.delete(
            "dashboard_id",
        )
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: SGPClient) -> None:
        response = client.evaluation_dashboards.with_raw_response.delete(
            "dashboard_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluation_dashboard = response.parse()
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: SGPClient) -> None:
        with client.evaluation_dashboards.with_streaming_response.delete(
            "dashboard_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluation_dashboard = response.parse()
            assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dashboard_id` but received ''"):
            client.evaluation_dashboards.with_raw_response.delete(
                "",
            )


class TestAsyncEvaluationDashboards:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        evaluation_dashboard = await async_client.evaluation_dashboards.create(
            name="x",
        )
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        evaluation_dashboard = await async_client.evaluation_dashboards.create(
            name="x",
            description="description",
            evaluation_group_id="evaluation_group_id",
            evaluation_id="evaluation_id",
            tags=["string"],
            template_dashboard_id="template_dashboard_id",
            widget_order=["string"],
        )
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_dashboards.with_raw_response.create(
            name="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluation_dashboard = await response.parse()
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_dashboards.with_streaming_response.create(
            name="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluation_dashboard = await response.parse()
            assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        evaluation_dashboard = await async_client.evaluation_dashboards.retrieve(
            dashboard_id="dashboard_id",
        )
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncSGPClient) -> None:
        evaluation_dashboard = await async_client.evaluation_dashboards.retrieve(
            dashboard_id="dashboard_id",
            include_archived=True,
            views=["widgets"],
        )
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_dashboards.with_raw_response.retrieve(
            dashboard_id="dashboard_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluation_dashboard = await response.parse()
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_dashboards.with_streaming_response.retrieve(
            dashboard_id="dashboard_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluation_dashboard = await response.parse()
            assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dashboard_id` but received ''"):
            await async_client.evaluation_dashboards.with_raw_response.retrieve(
                dashboard_id="",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncSGPClient) -> None:
        evaluation_dashboard = await async_client.evaluation_dashboards.update(
            dashboard_id="dashboard_id",
        )
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncSGPClient) -> None:
        evaluation_dashboard = await async_client.evaluation_dashboards.update(
            dashboard_id="dashboard_id",
            description="description",
            name="x",
            tags=["string"],
            widget_order=["string"],
        )
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_dashboards.with_raw_response.update(
            dashboard_id="dashboard_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluation_dashboard = await response.parse()
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_dashboards.with_streaming_response.update(
            dashboard_id="dashboard_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluation_dashboard = await response.parse()
            assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dashboard_id` but received ''"):
            await async_client.evaluation_dashboards.with_raw_response.update(
                dashboard_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        evaluation_dashboard = await async_client.evaluation_dashboards.list()
        assert_matches_type(AsyncCursorPage[EvaluationDashboard], evaluation_dashboard, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        evaluation_dashboard = await async_client.evaluation_dashboards.list(
            created_by_ids=["string"],
            ending_before="ending_before",
            evaluation_group_id="evaluation_group_id",
            evaluation_id="evaluation_id",
            include_archived=True,
            limit=1,
            search="search",
            sort_by="sort_by",
            sort_order="asc",
            starting_after="starting_after",
            tags=["string"],
        )
        assert_matches_type(AsyncCursorPage[EvaluationDashboard], evaluation_dashboard, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_dashboards.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluation_dashboard = await response.parse()
        assert_matches_type(AsyncCursorPage[EvaluationDashboard], evaluation_dashboard, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_dashboards.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluation_dashboard = await response.parse()
            assert_matches_type(AsyncCursorPage[EvaluationDashboard], evaluation_dashboard, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncSGPClient) -> None:
        evaluation_dashboard = await async_client.evaluation_dashboards.delete(
            "dashboard_id",
        )
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_dashboards.with_raw_response.delete(
            "dashboard_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluation_dashboard = await response.parse()
        assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_dashboards.with_streaming_response.delete(
            "dashboard_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluation_dashboard = await response.parse()
            assert_matches_type(EvaluationDashboard, evaluation_dashboard, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dashboard_id` but received ''"):
            await async_client.evaluation_dashboards.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from .dataset import Dataset
from .._models import BaseModel
from .shared.identity import Identity

__all__ = ["Evaluation", "Progress", "ProgressItems", "ProgressItemsFailedItem", "ProgressWorkflows"]


class ProgressItemsFailedItem(BaseModel):
    item_id: str

    error: Optional[str] = None

    error_type: Optional[str] = None


class ProgressItems(BaseModel):
    failed: int

    pending: int

    successful: int

    total: int

    failed_items: Optional[List[ProgressItemsFailedItem]] = None


class ProgressWorkflows(BaseModel):
    completed: int

    failed: int

    pending: int

    total: int


class Progress(BaseModel):
    """Progress of the evaluation's underlying async job"""

    items: Optional[ProgressItems] = None

    workflows: Optional[ProgressWorkflows] = None


class Evaluation(BaseModel):
    id: str

    created_at: datetime

    created_by: Identity
    """The identity that created the entity."""

    datasets: List[Dataset]

    name: str

    status: Literal["failed", "completed", "running"]

    tags: List[str]
    """The tags associated with the entity"""

    archived_at: Optional[datetime] = None

    description: Optional[str] = None

    error_count: Optional[int] = None
    """Number of items with task errors. Only populated on GET single evaluation."""

    metadata: Optional[Dict[str, object]] = None
    """Metadata key-value pairs for the evaluation"""

    object: Optional[Literal["evaluation"]] = None

    progress: Optional[Progress] = None
    """Progress of the evaluation's underlying async job"""

    status_reason: Optional[str] = None
    """Reason for evaluation status"""

    tasks: Optional[List["EvaluationTask"]] = None
    """Tasks executed during evaluation. Populated with optional `task` view."""


from .evaluation_task import EvaluationTask



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional

from .._models import BaseModel

__all__ = ["VectorStoreQueryResponse", "Metadata", "Vector"]


class Metadata(BaseModel):
    """Query execution metadata"""

    embedding_model: str
    """Embedding model used for query vectorization"""

    search_type: str
    """Type of search performed (semantic, lexical, hybrid)"""

    total_query_time_ms: int
    """Total end-to-end query execution time in milliseconds"""

    embedding_time_ms: Optional[int] = None
    """Time spent generating embeddings in milliseconds (None for lexical queries)"""

    index_query_time_ms: Optional[int] = None
    """Time spent querying the vector index (OpenSearch) in milliseconds"""

    reranking_model: Optional[str] = None
    """Reranking model used (None if reranking not enabled)"""

    reranking_time_ms: Optional[int] = None
    """Time spent reranking results in milliseconds (None if reranking not enabled)"""


class Vector(BaseModel):
    """A document in list/query responses."""

    id: str
    """Document ID"""

    metadata: Optional[Dict[str, object]] = None
    """Key-value metadata"""

    score: Optional[float] = None
    """Similarity score (for query results)"""

    text: Optional[str] = None
    """Text content"""

    vector: Optional[List[float]] = None
    """Embedding vector (if requested)"""


class VectorStoreQueryResponse(BaseModel):
    """Response for query operation."""

    metadata: Metadata
    """Query execution metadata"""

    vectors: List[Vector]
    """Array of matching documents"""



import logging
from queue import Empty, Queue
from typing import List
from unittest.mock import Mock, patch

import pytest

from scale_gp_beta import SGPClient
from scale_gp_beta._exceptions import APIError
from scale_gp_beta.lib.tracing.span import Span
from scale_gp_beta.lib.tracing.exceptions import ParamsCreationError
from scale_gp_beta.lib.tracing.trace_exporter import TraceExporter
from scale_gp_beta.types.span_upsert_batch_params import Item as SpanCreateRequest


def create_test_span_request(id_str: str) -> SpanCreateRequest:
    return SpanCreateRequest(name=id_str, id=id_str, trace_id= "trace-id", group_id="group-id", start_timestamp="start")


@pytest.fixture
def mock_sgp_client() -> Mock:
    client = Mock(spec=SGPClient)
    client.spans = Mock()
    client.spans.upsert_batch = Mock()
    return client


@pytest.fixture
def mock_span() -> Mock:
    span = Mock(spec=Span)
    span.span_id = "default_span_id"
    span.to_request_params.return_value = create_test_span_request("default_span")
    return span


def fill_queue(q: "Queue[Span]", num_spans: int) -> "list[SpanCreateRequest]":
    expected_requests: List[SpanCreateRequest] = []
    for i in range(num_spans):
        span_id = f"span_{i}"
        mock_s = Mock(spec=Span)
        request_param = create_test_span_request(span_id)
        mock_s.to_request_params.return_value = request_param
        q.put(mock_s)
        expected_requests.append(request_param)
    return expected_requests


class TestTraceExporterCreateBatches:
    def test_empty_queue(self):
        exporter = TraceExporter(max_batch_size=5, max_retries=3)
        q: Queue[Span] = Queue()
        batches = exporter._create_batches(q)
        assert batches == []
        assert q.empty()

    def test_less_than_max_batch_size(self):
        exporter = TraceExporter(max_batch_size=5, max_retries=3)
        q: Queue[Span] = Queue()
        expected_requests = fill_queue(q, 3)
        batches = exporter._create_batches(q)
        assert len(batches) == 1
        assert batches[0] == expected_requests
        assert q.empty()

    def test_exact_max_batch_size(self):
        exporter = TraceExporter(max_batch_size=3, max_retries=3)
        q: Queue[Span] = Queue()
        expected_requests = fill_queue(q, 3)
        batches = exporter._create_batches(q)
        assert len(batches) == 1
        assert batches[0] == expected_requests
        assert q.empty()

    def test_multiple_batches(self):
        exporter = TraceExporter(max_batch_size=2, max_retries=3)
        q: Queue[Span] = Queue()
        expected_requests = fill_queue(q, 5)

        batches = exporter._create_batches(q)
        assert len(batches) == 3
        assert batches[0] == expected_requests[0:2]
        assert batches[1] == expected_requests[2:4]
        assert batches[2] == [expected_requests[4]]
        assert q.empty()

    def test_max_batch_size_one(self):
        exporter = TraceExporter(max_batch_size=1, max_retries=3)
        q: Queue[Span] = Queue()
        expected_requests = fill_queue(q, 3)

        batches = exporter._create_batches(q)
        assert len(batches) == 3
        assert batches[0] == [expected_requests[0]]
        assert batches[1] == [expected_requests[1]]
        assert batches[2] == [expected_requests[2]]
        assert q.empty()

    def test_queue_becomes_empty_mid_batch(self):
        exporter = TraceExporter(max_batch_size=5, max_retries=3)
        q: Queue[Span] = Queue()
        initial_spans = [Mock(spec=Span) for _ in range(2)]
        expected_requests: List[SpanCreateRequest] = []
        for i, s in enumerate(initial_spans):
            req = create_test_span_request(f"s_{i}")
            s.to_request_params.return_value = req
            expected_requests.append(req)

        side_effect_list = [initial_spans[0], initial_spans[1], Empty]
        q.get_nowait = Mock(side_effect=side_effect_list)
        q.qsize = Mock(side_effect=[2, 1, 0, 0, 0])

        batches = exporter._create_batches(q)
        assert len(batches) == 1
        assert batches[0] == expected_requests
        assert q.get_nowait.call_count == 2


@patch("scale_gp_beta.lib.tracing.trace_exporter.time.sleep")
class TestTraceExporterExportBatch:
    def test_export_batch_success_first_try(self, mock_sleep: Mock, mock_sgp_client: Mock, caplog: pytest.LogCaptureFixture):
        exporter = TraceExporter(max_batch_size=5, max_retries=3)
        batch_data = [create_test_span_request("span1")]

        with caplog.at_level(logging.INFO):
            exporter._export_batch(batch_data, mock_sgp_client)

        mock_sgp_client.spans.upsert_batch.assert_called_once_with(items=batch_data)
        mock_sleep.assert_not_called()
        assert "API error" not in caplog.text
        assert "Failed to export span batch" not in caplog.text

    def test_export_batch_success_after_retries(self, mock_sleep: Mock, mock_sgp_client: Mock, caplog: pytest.LogCaptureFixture):
        exporter = TraceExporter(max_batch_size=5, max_retries=3)
        batch_data = [create_test_span_request("span1")]

        # Simulate APIError for the first two calls, then success
        mock_sgp_client.spans.upsert_batch.side_effect = [
            APIError("Simulated API Error 1", request=Mock(), body=None),
            APIError("Simulated API Error 2", request=Mock(), body=None),
            None,  # Success
        ]

        with caplog.at_level(logging.WARNING):
            exporter._export_batch(batch_data, mock_sgp_client)

        assert mock_sgp_client.spans.upsert_batch.call_count == 3
        mock_sgp_client.spans.upsert_batch.assert_called_with(items=batch_data)
        assert mock_sleep.call_count == 2
        assert caplog.text.count("API error occurred while exporting batch:") == 2
        assert "Failed to export span batch" not in caplog.text

    def test_export_batch_fails_all_retries(self, mock_sleep: Mock, mock_sgp_client: Mock, caplog: pytest.LogCaptureFixture):
        max_r = 2
        exporter = TraceExporter(max_batch_size=5, max_retries=max_r)
        batch_data = [create_test_span_request("span1")]

        mock_sgp_client.spans.upsert_batch.side_effect = APIError(
            "Simulated Persistent API Error", request=Mock(), body=Mock()
        )

        with caplog.at_level(logging.WARNING):
            exporter._export_batch(batch_data, mock_sgp_client)

        assert mock_sgp_client.spans.upsert_batch.call_count == max_r
        assert mock_sleep.call_count == max_r - 1
        assert caplog.text.count("API error occurred while exporting batch:") == max_r
        assert f"Failed to export span batch after {max_r} attempts" in caplog.text

    def test_export_batch_max_retries_one(self, mock_sleep: Mock, mock_sgp_client: Mock, caplog: pytest.LogCaptureFixture):
        exporter = TraceExporter(max_batch_size=5, max_retries=1)
        batch_data = [create_test_span_request("span1")]
        mock_sgp_client.spans.upsert_batch.side_effect = APIError("Simulated API Error", request=Mock(), body=Mock())

        with caplog.at_level(logging.ERROR):
            exporter._export_batch(batch_data, mock_sgp_client)

        mock_sgp_client.spans.upsert_batch.assert_called_once_with(items=batch_data)
        mock_sleep.assert_not_called()
        assert "Failed to export span batch after 1 attempts" in caplog.text


class TestTraceExporterExportIntegration:
    @patch("scale_gp_beta.lib.tracing.trace_exporter.TraceExporter._export_batch")
    @patch("scale_gp_beta.lib.tracing.trace_exporter.TraceExporter._create_batches")
    def test_export_orchestration_no_batches(
        self, mock_create_batches: Mock, mock_export_batch_method: Mock, mock_sgp_client: Mock, caplog: pytest.LogCaptureFixture
    ):
        mock_create_batches.return_value = []
        exporter = TraceExporter(max_batch_size=5, max_retries=3)
        q: Queue[Span] = Queue()

        with caplog.at_level(logging.DEBUG):
            exporter.export(mock_sgp_client, q)

        mock_create_batches.assert_called_once_with(q)
        mock_export_batch_method.assert_not_called()
        assert "No new span batches to export" in caplog.text

    @patch("scale_gp_beta.lib.tracing.trace_exporter.TraceExporter._export_batch")
    @patch("scale_gp_beta.lib.tracing.trace_exporter.TraceExporter._create_batches")
    def test_export_orchestration_multiple_batches(
        self, mock_create_batches: Mock, mock_export_batch_method: Mock, mock_sgp_client: Mock, caplog: pytest.LogCaptureFixture
    ):
        batch1_data = [create_test_span_request("s1")]
        batch2_data = [create_test_span_request("s2")]
        mock_create_batches.return_value = [batch1_data, batch2_data]

        exporter = TraceExporter(max_batch_size=5, max_retries=3)
        q: Queue[Span] = Queue()  # Queue content doesn't matter as _create_batches is mocked

        with caplog.at_level(logging.INFO):
            exporter.export(mock_sgp_client, q)

        mock_create_batches.assert_called_once_with(q)
        assert mock_export_batch_method.call_count == 2
        mock_export_batch_method.assert_any_call(batch1_data, mock_sgp_client)
        mock_export_batch_method.assert_any_call(batch2_data, mock_sgp_client)
        assert "Exporting 2 span batches" in caplog.text

    def test_export_e2e_success(self, mock_sgp_client: Mock, caplog: pytest.LogCaptureFixture):
        exporter = TraceExporter(max_batch_size=2, max_retries=1)
        q: Queue[Span] = Queue()
        num_spans = 3
        fill_queue(q, num_spans)

        with caplog.at_level(logging.INFO):
            exporter.export(mock_sgp_client, q)

        assert mock_sgp_client.spans.upsert_batch.call_count == 2

        first_call_args = mock_sgp_client.spans.upsert_batch.call_args_list[0]
        second_call_args = mock_sgp_client.spans.upsert_batch.call_args_list[1]

        assert len(first_call_args[1]["items"]) == 2
        assert len(second_call_args[1]["items"]) == 1

        assert "Exporting 2 span batches" in caplog.text
        assert "Failed to export" not in caplog.text

    def test_export_e2e_one_batch_fails(self, mock_sgp_client: Mock, caplog: pytest.LogCaptureFixture):
        exporter = TraceExporter(max_batch_size=1, max_retries=1)
        q: Queue[Span] = Queue()
        expected_requests = fill_queue(q, 2)  # Two batches of 1: [s0], [s1]

        # First batch succeeds, second fails
        mock_sgp_client.spans.upsert_batch.side_effect = [
            None,
            APIError("Failure on second batch", request=Mock(), body=Mock()),
        ]

        with caplog.at_level(logging.INFO):
            exporter.export(mock_sgp_client, q)

        assert mock_sgp_client.spans.upsert_batch.call_count == 2
        mock_sgp_client.spans.upsert_batch.assert_any_call(items=[expected_requests[0]])
        mock_sgp_client.spans.upsert_batch.assert_any_call(items=[expected_requests[1]])

        text: str = caplog.text
        assert "Exporting 2 span batches" in text
        assert "Failed to export span batch after 1 attempts" in text
        assert text.count("API error occurred while exporting batch: Failure on second batch") == 1


@patch("scale_gp_beta.lib.tracing.trace_exporter.ParamsCreationError", ParamsCreationError)
class TestTraceExporterSingleSpan:
    def test_create_one_span_batch_success(self, mock_span: Mock):
        exporter = TraceExporter(max_batch_size=5, max_retries=3)
        expected_request = mock_span.to_request_params()
        mock_span.reset_mock()

        batch = exporter._create_one_span_batch(mock_span)

        assert batch == [expected_request]
        mock_span.to_request_params.assert_called_once()

    def test_create_one_span_batch_param_creation_error(self, caplog: pytest.LogCaptureFixture):
        exporter = TraceExporter(max_batch_size=5, max_retries=3)
        mock_span_failing = Mock(spec=Span)
        error_message = "Failed to create params due to missing attribute"
        mock_span_failing.to_request_params.side_effect = ParamsCreationError(error_message)

        with caplog.at_level(logging.WARNING):
            batch = exporter._create_one_span_batch(mock_span_failing)

        assert batch == []
        assert f"ParamsCreationError:" in caplog.text
        assert error_message in caplog.text
        assert "dropping" in caplog.text

    @patch("scale_gp_beta.lib.tracing.trace_exporter.TraceExporter._export_batch")
    def test_export_span_orchestration(
            self, mock_export_batch: Mock, mock_sgp_client: Mock, mock_span: Mock, caplog: pytest.LogCaptureFixture
    ):
        exporter = TraceExporter(max_batch_size=5, max_retries=3)
        expected_request = mock_span.to_request_params()
        mock_span.reset_mock()
        expected_batch = [expected_request]

        with caplog.at_level(logging.INFO):
            exporter.export_span(mock_sgp_client, mock_span)

        mock_span.to_request_params.assert_called_once()
        mock_export_batch.assert_called_once_with(expected_batch, mock_sgp_client)
        assert f"Exporting single span with id {mock_span.span_id}" in caplog.text

    def test_export_span_e2e_success(self, mock_sgp_client: Mock, mock_span: Mock, caplog: pytest.LogCaptureFixture):
        exporter = TraceExporter(max_batch_size=5, max_retries=3)
        expected_request = mock_span.to_request_params()
        expected_batch = [expected_request]

        with caplog.at_level(logging.INFO):
            exporter.export_span(mock_sgp_client, mock_span)

        mock_sgp_client.spans.upsert_batch.assert_called_once_with(items=expected_batch)
        assert f"Exporting single span with id {mock_span.span_id}" in caplog.text
        assert "Failed to export" not in caplog.text



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .widgets import (
    WidgetsResource,
    AsyncWidgetsResource,
    WidgetsResourceWithRawResponse,
    AsyncWidgetsResourceWithRawResponse,
    WidgetsResourceWithStreamingResponse,
    AsyncWidgetsResourceWithStreamingResponse,
)
from .evaluation_dashboards import (
    EvaluationDashboardsResource,
    AsyncEvaluationDashboardsResource,
    EvaluationDashboardsResourceWithRawResponse,
    AsyncEvaluationDashboardsResourceWithRawResponse,
    EvaluationDashboardsResourceWithStreamingResponse,
    AsyncEvaluationDashboardsResourceWithStreamingResponse,
)

__all__ = [
    "WidgetsResource",
    "AsyncWidgetsResource",
    "WidgetsResourceWithRawResponse",
    "AsyncWidgetsResourceWithRawResponse",
    "WidgetsResourceWithStreamingResponse",
    "AsyncWidgetsResourceWithStreamingResponse",
    "EvaluationDashboardsResource",
    "AsyncEvaluationDashboardsResource",
    "EvaluationDashboardsResourceWithRawResponse",
    "AsyncEvaluationDashboardsResourceWithRawResponse",
    "EvaluationDashboardsResourceWithStreamingResponse",
    "AsyncEvaluationDashboardsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["FileImportFromCloudParams", "File"]


class FileImportFromCloudParams(TypedDict, total=False):
    files: Required[Iterable[File]]
    """List of files to import from cloud storage"""


class File(TypedDict, total=False):
    container: Required[str]
    """The cloud storage container/bucket name"""

    file_type: Required[str]
    """The MIME type of the file"""

    filename: Required[str]
    """The name of the file"""

    filepath: Required[str]
    """The path to the file within the container"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from .assessment_type import AssessmentType

__all__ = ["SpanAssessmentListParams"]


class SpanAssessmentListParams(TypedDict, total=False):
    assessment_type: AssessmentType
    """Filter by assessment type"""

    span_id: str
    """Filter by span ID.

    Either span_id or trace_id must be provided as a query parameter.
    """

    trace_id: str
    """Filter by trace ID.

    Either span_id or trace_id must be provided as a query parameter.
    """



from __future__ import annotations

import logging
from copy import deepcopy
from typing import TYPE_CHECKING, Type, Optional
from threading import RLock
from typing_extensions import override

from scale_gp_beta.types import SpanType, SpanStatus
from scale_gp_beta.types.span_upsert_batch_params import Item as SpanCreateRequest

from .util import iso_timestamp, generate_span_id
from .scope import Scope
from .types import SpanInputParam, SpanOutputParam, SpanMetadataParam
from .exceptions import ParamsCreationError

if TYPE_CHECKING:
    import contextvars
    from types import TracebackType

    from .trace_queue_manager import TraceQueueManager

log: logging.Logger = logging.getLogger(__name__)


class BaseSpan:
    """Base class for all span types, providing common attributes and context management.

    A span represents a single unit of work or operation within a trace. This base
    class defines the core interface and properties for spans, such as name, IDs,
    timestamps, and methods for starting and ending the span's lifecycle.

    It is intended to be subclassed by concrete span implementations like `Span`
    (for active tracing) and `NoOpSpan` (for when tracing is disabled).

    Attributes:
        name (str): The human-readable name of the span.
        trace_id (str): The ID of the trace this span belongs to.
        span_id (str): The unique ID of this span.
        parent_span_id (Optional[str]): The ID of the parent span, if this is a child span.
        start_time (Optional[str]): ISO 8601 timestamp of when the span started.
                                     Set by the `start()` method.
        end_time (Optional[str]): ISO 8601 timestamp of when the span ended.
                                   Set by the `end()` method.
        input (Optional[dict[str, Any]]): Input data or parameters for the span's operation.
        output (Optional[dict[str, Any]]): Output data or results from the span's operation.
        metadata (Optional[dict[str, Any]]): Additional arbitrary key-value metadata for the span.
        _contextvar_token (Optional[contextvars.Token]): Token for managing the span's presence
                                                        in the current execution context.
    """

    def __init__(
        self,
        name: str,
        trace_id: Optional[str] = None,
        queue_manager: Optional[TraceQueueManager] = None,
        span_id: Optional[str] = None,
        parent_span_id: Optional[str] = None,
        group_id: Optional[str] = None,
        input: Optional[SpanInputParam] = None,
        output: Optional[SpanOutputParam] = None,
        metadata: Optional[SpanMetadataParam] = None,
        span_type: SpanType = "STANDALONE"
    ):
        self._name = name
        self._trace_id: str = trace_id or "no_trace_id"
        self._group_id = group_id
        self._span_id: str = span_id or generate_span_id()
        self._parent_span_id = parent_span_id
        self._start_time: Optional[str] = None
        self._end_time: Optional[str] = None
        self._input: SpanInputParam = input or {}
        self._output: SpanOutputParam = output or {}
        self._metadata: SpanMetadataParam = metadata or {}
        self._span_type: SpanType = span_type
        self._status: SpanStatus = "SUCCESS"
        self._queue_manager = queue_manager

        self._contextvar_token: Optional[contextvars.Token[Optional[BaseSpan]]] = None
        self._lock = RLock()

    def start(self) -> None:
        pass

    def end(self) -> None:
        pass

    def flush(self, blocking: bool = True) -> None:
        pass

    @property
    def name(self) -> str:
        return self._name

    @property
    def trace_id(self) -> str:
        return self._trace_id

    @property
    def group_id(self) -> Optional[str]:
        return self._group_id

    @property
    def span_id(self) -> str:
        return self._span_id

    @property
    def parent_span_id(self) -> Optional[str]:
        return self._parent_span_id

    @property
    def status(self) -> SpanStatus:
        return self._status

    @property
    def span_type(self) -> SpanType:
        return self._span_type

    # with setters
    @property
    def start_time(self) -> Optional[str]:
        return self._start_time

    @start_time.setter
    def start_time(self, value: Optional[str]) -> None:
        with self._lock:
            self._start_time = value

    @property
    def end_time(self) -> Optional[str]:
        return self._end_time

    @end_time.setter
    def end_time(self, value: Optional[str]) -> None:
        with self._lock:
            self._end_time = value

    @property
    def metadata(self) -> SpanMetadataParam:
        return self._metadata

    @metadata.setter
    def metadata(self, value: SpanMetadataParam) -> None:
        # this does not protect against span.metadata["foo"] = "bar" which uses the getter, ditto input and output
        with self._lock:
            self._metadata = value

    @property
    def input(self) -> SpanInputParam:
        return self._input

    @input.setter
    def input(self, value: SpanInputParam) -> None:
        with self._lock:
            self._input = value

    @property
    def output(self) -> SpanOutputParam:
        return self._output

    @output.setter
    def output(self, value: SpanOutputParam) -> None:
        with self._lock:
            self._output = value

    def set_error(
            self,
            error_type: Optional[str] = None,
            error_message: Optional[str] = None,
            exception: Optional[BaseException] = None,
    ) -> None:
        # Naively record details in metadata for now, note that error capture only supported in context manager
        with self._lock:
            exception_type = type(exception).__name__ if exception else None
            exception_message = str(exception) if exception else None
            self._status = "ERROR"
            self.metadata["error"] = True
            self.metadata["error_type"] = error_type or exception_type
            self.metadata["error_message"] = error_message or exception_message

    def __enter__(self) -> BaseSpan:
        self.start()
        return self

    def __exit__(
            self,
            exc_type: Optional[Type[BaseException]],
            exc_val: Optional[BaseException],
            exc_tb: Optional[TracebackType]
    ) -> None:
        # TODO: support error observations when using direct span.start() and span.end()
        if exc_type is not None:
            self.set_error(exception=exc_val)
        self.end()

    def to_request_params(self) -> SpanCreateRequest:
        with self._lock:
            if self.start_time is None:
                raise ParamsCreationError("No start time specified")

            request_data = SpanCreateRequest(
                name=self.name,
                id=self.span_id,
                trace_id=self.trace_id,
                start_timestamp=self.start_time,
                input=self.input,
                output=self.output,
                metadata=self.metadata,
                status=self.status,
                type=self.span_type
            )

            if self.end_time is not None:
                request_data["end_timestamp"] = self.end_time

            # parent_span_id is optional (root spans)
            if self.parent_span_id is not None:
                request_data["parent_id"] = self.parent_span_id

            if self.group_id is not None:
                request_data["group_id"] = self.group_id

            # ensure no future changes to metadata, input or output changes request_data, full isolation
            request_data = deepcopy(request_data)
            return request_data

    @override
    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}("
            f"name='{self.name}', "
            f"span_id='{self.span_id}', "
            f"trace_id='{self.trace_id}', "
            f"parent_span_id='{self.parent_span_id}', "
            f"group_id='{self.group_id}', "
            f"start_time='{self.start_time}', "
            f"end_time='{self.end_time}', "
            f"input='{self.input}', "
            f"output='{self.output}', "
            f"metadata='{self.metadata}', "
            f"span_type='{self.span_type}', "
            f"status='{self.status}'"
            ")"
        )

    @override
    def __str__(self) -> str:
        return self.__repr__()


class NoOpSpan(BaseSpan):
    @override
    def start(self) -> None:
        if self.start_time is not None:
            log.warning(f"Span {self.name}: {self.span_id} has already started at {self.start_time}")
            return

        self.start_time = iso_timestamp()
        self._contextvar_token = Scope.set_current_span(self)

    @override
    def end(self) -> None:
        if self.end_time is not None:
            log.warning(f"Span {self.name}: {self.span_id} has already ended at {self.end_time}")
            return

        if self._contextvar_token is None:
            log.warning(f"Span {self.name}: {self.span_id} has not started yet.")
            return

        self.end_time = iso_timestamp()
        Scope.reset_current_span(self._contextvar_token)
        self._contextvar_token = None


class Span(BaseSpan):
    """An operational span implementation that records and reports tracing data.

    `Span` instances represent actual units of work that are part of an active trace.
    They record timestamps, manage their context in `Scope`, and interact with the
    `TraceQueueManager` to report their start and end events for later export.
    """

    def __init__(
        self,
        name: str,
        trace_id: str,
        queue_manager: TraceQueueManager,
        span_id: Optional[str] = None,
        parent_span_id: Optional[str] = None,
        group_id: Optional[str] = None,
        input: Optional[SpanInputParam] = None,
        output: Optional[SpanOutputParam] = None,
        metadata: Optional[SpanMetadataParam] = None,
        span_type: SpanType = "STANDALONE",
    ):
        super().__init__(name, trace_id, queue_manager, span_id, parent_span_id, group_id, input, output, metadata, span_type)
        self._queue_manager: TraceQueueManager = queue_manager
        self._trace_id: str = trace_id

    @override
    def flush(self, blocking: bool = True) -> None:
        """Export span. Defaults to in-thread and will block until the request is complete.

        With `blocking=False`, this method will enqueue the request for the background worker.
        The background worker batches and sends asynchronously.
        :param blocking:
        """
        if blocking:
            self._queue_manager.export_now(self)
        else:
            self._queue_manager.enqueue(self)

    @override
    def start(self) -> None:
        """Starts the operational Span.

        Sets the `start_time`, reports the span start to the `TraceQueueManager`
        , and registers this span as the current span.
        """
        with self._lock:
            if self.start_time is not None:
                log.warning(f"Span {self.name}: {self.span_id} has already started at {self.start_time}")
                return

            self.start_time = iso_timestamp()
            self._queue_manager.report_span_start(self)
            self._contextvar_token = Scope.set_current_span(self)

    @override
    def end(self) -> None:
        """Ends the operational Span.

        Sets the `end_time`, reports the span end (with its complete data) to the
        `TraceQueueManager` for queuing and export, and resets this span from the
        `Scope`.
        """
        with self._lock:
            if self.end_time is not None:
                log.warning(f"Span {self.name}: {self.span_id} has already ended at {self.end_time}")
                return
            if self._contextvar_token is None:
                log.warning(
                    (
                        f"Span {self.name}: {self.span_id} attempting to end without a valid context token. "
                        "Was start() called and completed successfully?"
                    )
                )
                return

            self.end_time = iso_timestamp()
            self._queue_manager.report_span_end(self)
            Scope.reset_current_span(self._contextvar_token)
            self._contextvar_token = None
